"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preprocess = exports.getTypeReference = exports.getTypeReferenceExpr = exports.getMapDescriptor = exports.setIdentifierForDependency = exports.resetDependencyMap = void 0;
const descriptor = require("./compiler/descriptor.js");
const ts = require("typescript");
const symbolMap = new Map();
const dependencyMap = new Map();
const mapMap = new Map();
function resetDependencyMap() {
    dependencyMap.clear();
}
exports.resetDependencyMap = resetDependencyMap;
function setIdentifierForDependency(dependency, identifier) {
    dependencyMap.set(dependency, identifier);
}
exports.setIdentifierForDependency = setIdentifierForDependency;
function isMapEntry(descriptor) {
    return descriptor?.options?.map_entry ?? false;
}
function getMapDescriptor(typeName) {
    return mapMap.get(typeName);
}
exports.getMapDescriptor = getMapDescriptor;
function getTypeReferenceExpr(rootDescriptor, typeName) {
    const path = symbolMap.get(typeName);
    if (!path || !dependencyMap.has(path)) {
        return ts.factory.createIdentifier(removeRootPackageName(typeName, rootDescriptor.package));
    }
    return ts.factory.createPropertyAccessExpression(dependencyMap.get(path), removeLeadingDot(typeName));
}
exports.getTypeReferenceExpr = getTypeReferenceExpr;
function getTypeReference(rootDescriptor, typeName) {
    const path = symbolMap.get(typeName);
    if (!path || !dependencyMap.has(path)) {
        return ts.factory.createTypeReferenceNode(removeRootPackageName(typeName, rootDescriptor.package));
    }
    return ts.factory.createTypeReferenceNode(ts.factory.createQualifiedName(dependencyMap.get(path), removeLeadingDot(typeName)));
}
exports.getTypeReference = getTypeReference;
function removeLeadingDot(name) {
    return name.replace(/^\./, "");
}
function replaceDoubleDots(name) {
    return name.replace(/\.\./g, ".");
}
function removeRootPackageName(name, packageName) {
    return removeLeadingDot(packageName ? name.replace(`${packageName}.`, "") : name);
}
function preprocess(targetDescriptor, path, prefix) {
    for (const enumDescriptor of targetDescriptor.enum_type) {
        symbolMap.set(replaceDoubleDots(`${prefix}.${enumDescriptor.name}`), path);
    }
    const messages = targetDescriptor instanceof descriptor.FileDescriptorProto
        ? targetDescriptor.message_type
        : targetDescriptor.nested_type;
    for (let index = messages.length - 1; index >= 0; index--) {
        const messageDescriptor = messages[index];
        const name = replaceDoubleDots(`${prefix}.${messageDescriptor.name}`);
        if (isMapEntry(messageDescriptor)) {
            mapMap.set(name, messageDescriptor);
            messages.splice(index, 1);
            continue;
        }
        symbolMap.set(name, path);
        preprocess(messageDescriptor, path, name);
    }
}
exports.preprocess = preprocess;
